@@ -1,88 +1,33 @@
-/* OV511 Decompression Support Module
+/* OV511 Decompression Support
  *
- * Copyright (c) 1999-2003 Mark W. McClelland. All rights reserved.
- * http://alpha.dyndns.org/ov511/
+ * Copyright (c) 1999-2006 Mark McClelland <mark@ovcam.org>
+ * http://ovcam.org/ov511/
  *
  * Original decompression code Copyright 1998-2000 OmniVision Technologies
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; version 2 of the License.
+ * Free Software Foundation; either version 2 of the License, or (at your
+ * option) any later version. NO WARRANTY OF ANY KIND is expressed or implied.
  */
 
+#define __NO_VERSION__
+
 #include <linux/config.h>
-
-#if defined(OUTSIDE_KERNEL)
-	#if defined(CONFIG_MODVERSIONS) && !defined(MODVERSIONS)
-		#define MODVERSIONS
-	#endif
-
-	#include <linux/version.h>
-
-	#ifdef MODVERSIONS
-		#include <linux/modversions.h>
-	#endif
-#else
-	#include <linux/version.h>
-#endif
-
-#include <linux/module.h>
-#include <linux/init.h>
+#include <linux/version.h>
 
 #include "ov511.h"
 
-/******************************************************************************
- * Version Information
- ******************************************************************************/
-
-#define DRIVER_VERSION "v1.6"
-#define DRIVER_AUTHOR "Mark McClelland <mark@alpha.dyndns.org>, OmniVision \
-Technologies <http://www.ovt.com/>"
-#define DRIVER_DESC "OV511 Decompression Module"
-
-/******************************************************************************
- * Prototypes
- ******************************************************************************/
-
-extern int ov511_register_decomp_module(int ver, struct ov51x_decomp_ops *ops,
-					int ov518, int mmx);
-extern void ov511_deregister_decomp_module(int ov518, int mmx);
-
-/******************************************************************************
- * Decompression Module Interface Constants
- ******************************************************************************/
-
-static const int interface_ver = DECOMP_INTERFACE_VER;
-static const int ov518 = 0;
-static const int mmx = 0;
-
-/******************************************************************************
- * Module Features
- ******************************************************************************/
-
-static int debug;
-
-MODULE_PARM(debug, "i");
-MODULE_PARM_DESC(debug,
-  "Debug level: 0=none, 1=inits, 2=warning, 3=config, 4=functions, 5=max");
-
-MODULE_AUTHOR(DRIVER_AUTHOR);
-MODULE_DESCRIPTION(DRIVER_DESC);
-#if defined(MODULE_LICENSE)	/* Introduced in ~2.4.10 */
-MODULE_LICENSE("GPL");
-#endif
-
-/******************************************************************************
- * Decompression Functions
- ******************************************************************************/
+/* Variables in ov511.c: */
+extern int debug;
 
 static void
-DecompressYHI(unsigned char *pIn,
-	      unsigned char *pOut,
-	      int           *iIn,	/* in/out */
-	      int           *iOut,	/* in/out */
-	      const int      w,
-	      const int      YUVFlag)
+decomp_8x8(unsigned char *pIn,
+	   unsigned char *pOut,
+	   int           *iIn,	/* in/out */
+	   int           *iOut,	/* in/out */
+	   const int      w,
+	   const int      YUVFlag)
 {
 	short ZigZag[64];
 	int temp[64];
@@ -241,9 +186,9 @@ DecompressYHI(unsigned char *pIn,
 		}
 	}
 
-	/*************
-	 * De-ZigZag *
-	 *************/
+	/****************************
+	 * De-ZigZag and Dequantize *
+	 ****************************/
 
 	for (j = 0; j < 64; j++) {
 		DeZigZag[j] = 0;
@@ -424,21 +369,26 @@ DecompressYHI(unsigned char *pIn,
 	*iOut = out_pos + 8;
 }
 
-#define DECOMP_Y() DecompressYHI(pIn, pY, &iIn, &iY, w, 1)
-#define DECOMP_U() DecompressYHI(pIn, pU, &iIn, &iU, w/2, 2)
-#define DECOMP_V() DecompressYHI(pIn, pV, &iIn, &iV, w/2, 2)
+#define DECOMP_Y() decomp_8x8(pIn, pY, &iIn, &iY, w, 1)
+#define DECOMP_U() decomp_8x8(pIn, pU, &iIn, &iU, w/2, 2)
+#define DECOMP_V() decomp_8x8(pIn, pV, &iIn, &iV, w/2, 2)
 
-inline static int
-Decompress400HiNoMMX(unsigned char *pIn,
-		     unsigned char *pOut,
-		     const int      w,
-		     const int      h,
-		     const int      inSize)
+/* Input format is raw iso data (with header and packet
+ * number stripped, and zero-padding removed).
+ * Output format is YUV400
+ * Returns uncompressed data length if success, or zero if error
+ */
+static int
+ov511_decomp_400(unsigned char *pIn,
+		 unsigned char *pOut,
+		 unsigned char *pTmp,
+		 int		w,
+		 int		h,
+		 int		inSize)
 {
 	unsigned char *pY = pOut;
-	int x, y, iIn, iY;
+	int x, y, iY, iIn = 0;
 
-	iIn = 0;
 	for (y = 0; y < h; y += 8) {
 		iY = w*y;
 
@@ -446,139 +396,60 @@ Decompress400HiNoMMX(unsigned char *pIn,
 			DECOMP_Y();
 	}
 
-	return 0;
+	return w * h;
 }
 
-inline static int
-Decompress420HiNoMMX(unsigned char *pIn,
-		     unsigned char *pOut,
-		     const int      w,
-		     const int      h,
-		     const int      inSize)
-{
-	unsigned char *pY = pOut;
-	unsigned char *pU = pY + w*h;
-	unsigned char *pV = pU + w*h/4;
-	int xY, xUV, iY, iU, iV, iIn, count;
-	const int nBlocks = (w*h) / (32*8);
-
-	iIn = 0;
-	iY = iU = iV = 0;
-	xY = xUV = 0;
-
-	for (count = 0; count < nBlocks; count++) {
-			DECOMP_U();
-			DECOMP_V();	xUV += 16;
-			if (xUV >= w) {
-				iU += (w*7)/2;
-				iV += (w*7)/2;
-				xUV = 0;
-			}
-
-			DECOMP_Y();	xY += 8;
-			DECOMP_Y();	xY += 8;
-			if (xY >= w) {
-				iY += w*7;
-				xY = 0;
-			}
-			DECOMP_Y();	xY += 8;
-			DECOMP_Y();	xY += 8;
-			if (xY >= w) {
-				iY += w*7;
-				xY = 0;
-			}
-	}
-
-	return 0;
-}
-
-/* Input format is raw isoc. data (with header and packet
- * number stripped, and all-zero blocks removed).
- * Output format is YUV400
- * Returns uncompressed data length if success, or zero if error
- */
-static int
-Decompress400(unsigned char *pIn,
-	      unsigned char *pOut,
-	      unsigned char *pTmp,
-	      int	     w,
-	      int	     h,
-	      int	     inSize)
-{
-	int numpix = w * h;
-	int rc;
-
-	PDEBUG(4, "%dx%d pIn=%p pOut=%p inSize=%d", w, h, pIn, pOut, inSize);
-
-	rc = Decompress400HiNoMMX(pIn, pOut, w, h, inSize);
-
-	if (rc)
-		return 0;
-
-	return numpix;
-}
-
-/* Input format is raw isoc. data (with header and packet
- * number stripped, and all-zero blocks removed).
+/* Input format is raw iso data (with header and packet
+ * number stripped, and zero-padding removed).
  * Output format is planar YUV420
  * Returns uncompressed data length if success, or zero if error
  */
 static int
-Decompress420(unsigned char *pIn,
-	      unsigned char *pOut,
-	      unsigned char *pTmp,
-	      int	     w,
-	      int	     h,
-	      int	     inSize)
+ov511_decomp_420(unsigned char *pIn,
+		 unsigned char *pOut,
+		 unsigned char *pTmp,
+		 int		w,
+		 int		h,
+		 int		inSize)
 {
 	int numpix = w * h;
-	int rc;
+	unsigned char *pY = pOut;
+	unsigned char *pU = pY + numpix;
+	unsigned char *pV = pU + numpix/4;
+	int b, xY = 0, xUV = 0, iY = 0, iU = 0, iV = 0, iIn = 0;
+	const int nBlocks = (w*h) / (32*8);
 
-	PDEBUG(4, "%dx%d pIn=%p pOut=%p inSize=%d", w, h, pIn, pOut, inSize);
+	for (b = 0; b < nBlocks; b++) {
+		DECOMP_U();
+		DECOMP_V();	xUV += 16;
 
-	rc = Decompress420HiNoMMX(pIn, pOut, w, h, inSize);
+		if (xUV >= w) {
+			iU += (w*7)/2;
+			iV += (w*7)/2;
+			xUV = 0;
+		}
 
-	if (rc)
-		return 0;
+		DECOMP_Y();	xY += 8;
+		DECOMP_Y();	xY += 8;
+
+		if (xY >= w) {
+			iY += w*7;
+			xY = 0;
+		}
+
+		DECOMP_Y();	xY += 8;
+		DECOMP_Y();	xY += 8;
+
+		if (xY >= w) {
+			iY += w*7;
+			xY = 0;
+		}
+	}
 
 	return (numpix * 3 / 2);
 }
 
-/******************************************************************************
- * Module Functions
- ******************************************************************************/
-
-static struct ov51x_decomp_ops decomp_ops = {
-	.decomp_400 =	Decompress400,
-	.decomp_420 =	Decompress420,
-	.owner =	THIS_MODULE,
+struct ov51x_decomp_ops ov511_decomp_ops = {
+	.decomp_400 =	ov511_decomp_400,
+	.decomp_420 =	ov511_decomp_420,
 };
-
-static int __init
-decomp_init(void)
-{
-	int rc;
-
-	EXPORT_NO_SYMBOLS;
-
-	rc = ov511_register_decomp_module(DECOMP_INTERFACE_VER, &decomp_ops,
-					  ov518, mmx);
-	if (rc) {
-		err("Could not register with ov511 (rc=%d)", rc);
-		return -1;
-	}
-
-	info(DRIVER_VERSION " : " DRIVER_DESC);
-
-	return 0;
-}
-
-static void __exit
-decomp_exit(void)
-{
-	ov511_deregister_decomp_module(ov518, mmx);
-	info("deregistered\n");
-}
-
-module_init(decomp_init);
-module_exit(decomp_exit);
