@@ -1,34 +1,21 @@
-/* OV518 Decompression Support Module (No-MMX version)
+/* OV518 Decompression Support
  *
- * Copyright (c) 2002-2003 Mark W. McClelland. All rights reserved.
- * http://alpha.dyndns.org/ov511/
+ * Copyright (c) 2002-2006 Mark McClelland <mark@ovcam.org>
+ * http://ovcam.org/ov511/
  *
  * Fast integer iDCT by Yuri van Oers <yvanoers AT xs4all.nl>
  * Original OV511 decompression code Copyright 1998-2000 OmniVision Technologies
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; version 2 of the License.
+ * Free Software Foundation; either version 2 of the License, or (at your
+ * option) any later version. NO WARRANTY OF ANY KIND is expressed or implied.
  */
 
+#define __NO_VERSION__
+
 #include <linux/config.h>
-
-#if defined(OUTSIDE_KERNEL)
-	#if defined(CONFIG_MODVERSIONS) && !defined(MODVERSIONS)
-		#define MODVERSIONS
-	#endif
-
-	#include <linux/version.h>
-
-	#ifdef MODVERSIONS
-		#include <linux/modversions.h>
-	#endif
-#else
-	#include <linux/version.h>
-#endif
-
-#include <linux/module.h>
-#include <linux/init.h>
+#include <linux/version.h>
 
 #include "ov511.h"
 
@@ -45,60 +32,24 @@
 #define PRINT_QT
 
 /******************************************************************************
- * Version Information
+ * Globals
  ******************************************************************************/
 
-#define DRIVER_VERSION "v1.3"
-#define DRIVER_AUTHOR "Mark McClelland <mark@alpha.dyndns.org>, \
-Yuri van Oers <yvanoers AT xs4all.nl>, OmniVision Technologies \
-<http://www.ovt.com/>"
-#define DRIVER_DESC "OV518 Decompression Module"
-
-/******************************************************************************
- * Decompression Module Interface Constants
- ******************************************************************************/
-
-static const int interface_ver = DECOMP_INTERFACE_VER;
-static const int ov518 = 1;
-static const int mmx = 0;
-
-/******************************************************************************
- * Module Features
- ******************************************************************************/
-
-static int debug = 0;
+extern int ov518_color;
 
 /* Static quantization. This uses a fixed quantization table versus the one
  * that is normally embedded in the data. Define this if you see very bad
  * contrast or "blockiness" in the decompressed output. */
 static int staticquant = 0;
 
-MODULE_PARM(debug, "i");
-MODULE_PARM_DESC(debug, 
-  "Debug level: 0=none, 1=inits, 2=warning, 3=config, 4=functions, 5=max");
-
-MODULE_AUTHOR(DRIVER_AUTHOR);
-MODULE_DESCRIPTION(DRIVER_DESC);
-#if defined(MODULE_LICENSE)	/* Introduced in ~2.4.10 */
-MODULE_LICENSE("GPL");
-#endif
-
-/******************************************************************************
- * Prototypes
- ******************************************************************************/
-
-extern int ov511_register_decomp_module(int ver, struct ov51x_decomp_ops *ops,
-					int ov518, int mmx);
-extern void ov511_deregister_decomp_module(int ov518, int mmx);
-
 /******************************************************************************
  * Local Data Types
  ******************************************************************************/
 
 /* Make sure this remains naturally aligned and 2^n bytes in size */
 struct tree_node {
-	short left;		/* Pointer to left child node */
-	short right;		/* Pointer to right child node */
+	short left;		/* Index of left child node */
+	short right;		/* Index of right child node */
 	signed char depth;	/* Depth (starting at 1) if leaf, else -1 */
 	signed char coeffbits;	/* Size of coefficient data, or zero if none */
 	signed char skip;	/* Number of zero coefficients. Unused w/ DC */
@@ -1230,11 +1181,11 @@ DCT_8x4(int *coeff, unsigned char *out)
  * determines the positin in the input buffer.
  */
 static int 
-decompress8x4(unsigned char	*pOut,
-	      unsigned char	*pIn,
-	      int		*lastDC,
-	      int		uvFlag,
-	      struct comp_info	*cinfo)
+decomp_8x4(unsigned char	*pOut,
+	   unsigned char	*pIn,
+	   int			*lastDC,
+	   int			 uvFlag,
+	   struct comp_info	*cinfo)
 {
 	int i, x, y, dc;
 	int coeffs[32];
@@ -1308,13 +1259,13 @@ copyBlock(unsigned char *src, unsigned char *dest, int destInc)
 }
 
 static inline int
-decompress400NoMMXOV518(unsigned char	 *pIn,
-			unsigned char	 *pOut,
-			unsigned char	 *pTmp,
-			const int	 w, 
-			const int	 h, 
-			const int	 numpix,
-			struct comp_info *cinfo)
+do_decomp_400(unsigned char	 *pIn,
+	      unsigned char	 *pOut,
+	      unsigned char	 *pTmp,
+	      const int		  w, 
+	      const int		  h, 
+	      const int		  numpix,
+	      struct comp_info	 *cinfo)
 {
 	int iOutY, x, y;
 	int lastYDC = 0;
@@ -1325,7 +1276,7 @@ decompress400NoMMXOV518(unsigned char	 *pIn,
 		iOutY = w * y;
 		x = 0;
 		do {
-			decompress8x4(pTmp, pIn, &lastYDC, 0, cinfo);
+			decomp_8x4(pTmp, pIn, &lastYDC, 0, cinfo);
 			copyBlock(pTmp, pOut + iOutY, w);
 			iOutY += 8;
 			x += 8;
@@ -1345,13 +1296,13 @@ decompress400NoMMXOV518(unsigned char	 *pIn,
 }
 
 static inline int
-decompress420NoMMXOV518(unsigned char	 *pIn,
-			unsigned char	 *pOut,
-			unsigned char	 *pTmp,
-			const int	 w, 
-			const int	 h, 
-			const int	 numpix,
-			struct comp_info *cinfo)
+do_decomp_420(unsigned char	 *pIn,
+	      unsigned char	 *pOut,
+	      unsigned char	 *pTmp,
+	      const int		  w, 
+	      const int		  h, 
+	      const int		  numpix,
+	      struct comp_info	 *cinfo)
 {
 	unsigned char *pOutU = pOut + numpix;
 	unsigned char *pOutV = pOutU + numpix / 4;
@@ -1368,30 +1319,28 @@ decompress420NoMMXOV518(unsigned char	 *pIn,
 
 		x = 0;
 		do {
-			decompress8x4(pTmp, pIn, &lastYDC, 0, cinfo);
+			decomp_8x4(pTmp, pIn, &lastYDC, 0, cinfo);
 			copyBlock(pTmp, pOut + iOutY, w);
 			iOutY += 8;
 			x += 8;
 		} while (x < w);
 
-
-
 		iOutY = w * (y + 4);
 		x = 0;
 		do {
-			decompress8x4(pTmp, pIn, &lastUDC, 1, cinfo);
+			decomp_8x4(pTmp, pIn, &lastUDC, 1, cinfo);
 			copyBlock(pTmp, pOutU + iOutU, w/2);
 			iOutU += 8;
 
-			decompress8x4(pTmp, pIn, &lastVDC, 1, cinfo);
+			decomp_8x4(pTmp, pIn, &lastVDC, 1, cinfo);
 			copyBlock(pTmp, pOutV + iOutV, w/2);
 			iOutV += 8;
 
-			decompress8x4(pTmp, pIn, &lastYDC, 0, cinfo);
+			decomp_8x4(pTmp, pIn, &lastYDC, 0, cinfo);
 			copyBlock(pTmp, pOut + iOutY, w);
 			iOutY += 8;
 
-			decompress8x4(pTmp, pIn, &lastYDC, 0, cinfo);
+			decomp_8x4(pTmp, pIn, &lastYDC, 0, cinfo);
 			copyBlock(pTmp, pOut + iOutY, w);
 			iOutY += 8;
 
@@ -1449,18 +1398,18 @@ get_qt_dynamic(unsigned char *pIn, struct comp_info *cinfo)
 	return 0;
 }
 
-/* Input format is raw isoc. data (with intact SOF header, packet numbers
- * stripped, and all-zero blocks removed).
+/* Input format is raw iso data (with intact SOF header, packet numbers
+ * stripped, and zero-padding removed).
  * Output format is planar YUV400
  * Returns uncompressed data length if success, or zero if error
  */
 static int 
-Decompress400(unsigned char *pIn,
-	      unsigned char *pOut,
-	      unsigned char *pTmp,
-	      int	     w,
-	      int	     h,
-	      int	     inSize)
+ov518_decomp_400(unsigned char *pIn,
+		 unsigned char *pOut,
+		 unsigned char *pTmp,
+		 int		w,
+		 int		h,
+		 int		inSize)
 {
 	struct comp_info cinfo;
 	int numpix = w * h;
@@ -1481,25 +1430,25 @@ Decompress400(unsigned char *pIn,
 	}
 
 	/* Decompress, skipping the 8-byte SOF header */
-	if (decompress400NoMMXOV518(pIn + 8, pOut, pTmp, w, h, numpix, &cinfo))
+	if (do_decomp_400(pIn + 8, pOut, pTmp, w, h, numpix, &cinfo))
 //		return 0;
 		; /* Don't return error yet */
 
 	return (numpix);
 }
 
-/* Input format is raw isoc. data (with intact SOF header, packet numbers
- * stripped, and all-zero blocks removed).
+/* Input format is raw iso data (with intact SOF header, packet numbers
+ * stripped, and zero-padding removed).
  * Output format is planar YUV420
  * Returns uncompressed data length if success, or zero if error
  */
 static int 
-Decompress420(unsigned char *pIn,
-	      unsigned char *pOut,
-	      unsigned char *pTmp,
-	      int	     w,
-	      int	     h,
-	      int	     inSize)
+ov518_decomp_420(unsigned char *pIn,
+		 unsigned char *pOut,
+		 unsigned char *pTmp,
+		 int		w,
+		 int		h,
+		 int		inSize)
 {
 	struct comp_info cinfo;
 	int numpix = w * h;
@@ -1520,50 +1469,25 @@ Decompress420(unsigned char *pIn,
 	}
 
 	/* Decompress, skipping the 8-byte SOF header */
-	if (decompress420NoMMXOV518(pIn + 8, pOut, pTmp, w, h, numpix, &cinfo))
+	if (do_decomp_420(pIn + 8, pOut, pTmp, w, h, numpix, &cinfo))
 //		return 0;
 		; /* Don't return error yet */
 
+	if (!ov518_color) {
+		int i;
+		unsigned char *pU = pOut + numpix;
+		unsigned char *pV = pU + numpix/4;
+
+		for (i = 0; i < numpix/4; i++) {
+			*pU++ = 127;
+			*pV++ = 127;
+		}
+	}
+
 	return (numpix * 3 / 2);
 }
 
-/******************************************************************************
- * Module Functions
- ******************************************************************************/
-
-static struct ov51x_decomp_ops decomp_ops = {
-	.decomp_400 =	Decompress400,	
-	.decomp_420 =	Decompress420,	
-	.owner =	THIS_MODULE,
+struct ov51x_decomp_ops ov518_decomp_ops = {
+	.decomp_400 =	ov518_decomp_400,	
+	.decomp_420 =	ov518_decomp_420,	
 };
-
-static int __init 
-decomp_init(void)
-{
-	int rc;
-
-	EXPORT_NO_SYMBOLS;
-
-	rc = ov511_register_decomp_module(DECOMP_INTERFACE_VER, &decomp_ops,
-					  ov518, mmx);
-	if (rc) {
-		err("Could not register with ov511 (rc=%d)", rc);
-		return -1;
-	}
-
-	info(DRIVER_VERSION " : " DRIVER_DESC);
-	PDEBUG(1, "Using %s, %s quantization", IDCT_MESSAGE, 
-	       staticquant ? "static" : "dynamic");
-
-	return 0;
-}
-
-static void __exit 
-decomp_exit(void)
-{
-	ov511_deregister_decomp_module(ov518, mmx);
-	info("deregistered");
-}
-
-module_init(decomp_init);
-module_exit(decomp_exit);
